package io.sixminutes.ridicule

import android.content.Context
import android.content.pm.PackageManager
import android.os.Build
import android.os.Build.VERSION
import android.telephony.TelephonyManager
import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage.LoadPackageParam
import java.lang.reflect.Method

class Hook : IXposedHookLoadPackage {

    @Throws(Throwable::class)
    override fun handleLoadPackage(lpparam: LoadPackageParam) {
        try {

            XposedHelpers.setStaticObjectField(Build::class.java, "MODEL", device_model)
            XposedHelpers.setStaticObjectField(Build::class.java, "MANUFACTURER", manufacturer)
            XposedHelpers.setStaticObjectField(Build::class.java, "BRAND", manufacturer)
            XposedHelpers.setStaticObjectField(VERSION::class.java, "RELEASE", os_version)
            XposedHelpers.setStaticIntField(VERSION::class.java, "SDK_INT", SDK_INT)


            //修改运营商
            hookTelephonyMethods(lpparam.classLoader)
            bypassPermissionChecks(lpparam.classLoader)

            // 修改Device ID
            XposedHelpers.findAndHookMethod(
                "android.telephony.TelephonyManager", lpparam.classLoader,
                "getDeviceId",
                object : XC_MethodHook() {
                    @Throws(Throwable::class)
                    protected override fun afterHookedMethod(param: MethodHookParam) {
                        XposedBridge.log("[$LOG_TAG] getDeviceId被调用")
                        // 修改 getDeviceId 方法的返回值
                        param.setResult(device_id)
                    }
                })

            // 修改广告ID
            XposedHelpers.findAndHookMethod(
                "com.google.android.gms.ads.identifier.AdvertisingIdClient\$Info",
                lpparam.classLoader,
                "getId",
                object : XC_MethodHook() {
                    protected override fun afterHookedMethod(param: MethodHookParam) {
                        XposedBridge.log("[$LOG_TAG] AdvertisingIdInfo.getId被调用")
                        // 返回伪造的广告ID
                        param.setResult(CUSTOM_ADVERTISING_ID)
                    }
                }
            )
            XposedHelpers.findAndHookMethod(
                "com.google.android.gms.ads.identifier.AdvertisingIdClient",
                lpparam.classLoader,
                "getAdvertisingIdInfo",
                Context::class.java,
                object : XC_MethodHook() {
                    protected override fun afterHookedMethod(param: MethodHookParam) {
                        try {
                            val info: Any = param.getResult() ?: return
                            XposedHelpers.findAndHookMethod(
                                info.javaClass,
                                "getId",
                                object : XC_MethodHook() {
                                    protected override fun afterHookedMethod(param: MethodHookParam) {
                                        param.setResult(CUSTOM_ADVERTISING_ID)
                                    }
                                }
                            )
                        } catch (e: Throwable) {
                            XposedBridge.log("Error in hooking getAdvertisingIdInfo: $e")
                        }
                    }
                }
            )
        } catch (e: Exception) {
            XposedBridge.log("修改设备型号时出错: " + e.message)
        }
    }


    companion object {
        private const val LOG_TAG = "LovinFaker"

        private const val screen_height = 2340
        private const val screen_width = 2340

        private const val device_model = "A001XM"
        private const val manufacturer = "Xiaomi"
        private const val os_version = "11"
        private const val SDK_INT = 30
        private const val device_id = "1ac1af0be39f7991"


        private const val country_code = "jp"

        private const val carrier_name = "SoftBank"
        private const val carrier_mcc = "44020"
        private const val CUSTOM_ADVERTISING_ID = "48442068-7132-4bd3-9349-707695891ce8"

    }

    private fun hookTelephonyMethods(classLoader: ClassLoader) {
        val tmClass = "android.telephony.TelephonyManager"

        // Hook getNetworkOperator()
        XposedHelpers.findAndHookMethod(
            tmClass,
            classLoader,
            "getNetworkOperator",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    param.setResult(carrier_mcc)
                    XposedBridge.log("[$LOG_TAG] getNetworkOperator被调用")
                }
            }
        )

        // Hook getNetworkOperatorName()
        XposedHelpers.findAndHookMethod(
            tmClass,
            classLoader,
            "getNetworkOperatorName",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    param.setResult(carrier_name)
                    XposedBridge.log("[$LOG_TAG] getNetworkOperatorName被调用")
                }
            }
        )

        // Hook getSimCountryIso()
        XposedHelpers.findAndHookMethod(
            tmClass,
            classLoader,
            "getSimCountryIso",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    param.setResult(country_code)
                    XposedBridge.log("[$LOG_TAG] getSimCountryIso被调用")
                }
            }
        )
    }

    private fun bypassPermissionChecks(classLoader: ClassLoader) {
        // 绕过 READ_PHONE_STATE 权限检查 (Android 12+)
        XposedHelpers.findAndHookMethod(
            "android.app.ContextImpl",
            classLoader,
            "checkPermission",
            String::class.java,
            Int::class.java,
            Int::class.java,
            object : XC_MethodHook() {
                override fun beforeHookedMethod(param: MethodHookParam) {
                    if (param.args[0] == android.Manifest.permission.READ_PHONE_STATE) {
                        param.setResult(PackageManager.PERMISSION_GRANTED)
                        XposedBridge.log("[$LOG_TAG] checkPermission被调用")
                    }
                }
            }
        )
    }
}