package io.sixminutes.ridicule

import android.util.Log
import java.io.DataOutputStream

object RootHelper {
    private const val TAG = "RootHelper"

    /**
     * 检查设备是否已 Root
     */
    fun isDeviceRooted(): Boolean {
        return try {
            Runtime.getRuntime().exec("su").outputStream.apply {
                write("exit\n".toByteArray())
                flush()
            }
            true
        } catch (e: Exception) {
            Log.e(TAG, "Root access not available", e)
            false
        }
    }

    /**
     * 请求 Root 权限
     */
    fun requestRootAccess(): Boolean {
        return try {
            val process = Runtime.getRuntime().exec("su")
            val outputStream = DataOutputStream(process.outputStream)
            outputStream.writeBytes("exit\n")
            outputStream.flush()
            process.waitFor() == 0
        } catch (e: Exception) {
            Log.e(TAG, "Failed to obtain root access", e)
            false
        }
    }

    /**
     * 执行 Root Shell 命令
     */
    fun runRootCommand(command: String): Boolean {
        return try {
            val process = Runtime.getRuntime().exec("su")
            val outputStream = DataOutputStream(process.outputStream)
            outputStream.writeBytes("$command\n")
            outputStream.writeBytes("exit\n")
            outputStream.flush()
            process.waitFor() == 0
        } catch (e: Exception) {
            Log.e(TAG, "Root command execution failed: $command", e)
            false
        }
    }
}
