package io.sixminutes.ridicule

import android.content.Context
import android.graphics.Color
import android.graphics.PixelFormat
import android.os.Handler
import android.view.Gravity
import android.view.View
import android.view.WindowManager

class TapIndicatorView(private val context: Context) {
    private var windowManager: WindowManager? = null
    private var tapView: View? = null

    /**
     * 显示点击指示器
     * @param x 点击位置 X 坐标
     * @param y 点击位置 Y 坐标
     */
    fun showTapIndicator(xx: Int, yy: Int) {
        if (tapView != null) return

        windowManager = context.getSystemService(Context.WINDOW_SERVICE) as WindowManager

        // 创建一个小圆点作为指示器
        tapView = View(context).apply {
            setBackgroundColor(Color.RED) // 红色小点
            alpha = 0.8f
        }

        val params = WindowManager.LayoutParams().apply {
            width = 50  // 指示器大小
            height = 50
            x = xx - 25  // 居中对齐
            y = yy - 25
            gravity = Gravity.TOP or Gravity.START
            format = PixelFormat.TRANSLUCENT
            type = WindowManager.LayoutParams.TYPE_APPLICATION_OVERLAY
            flags = WindowManager.LayoutParams.FLAG_NOT_FOCUSABLE or WindowManager.LayoutParams.FLAG_NOT_TOUCHABLE
        }

        windowManager?.addView(tapView, params)

        // 5 秒后移除
        Handler().postDelayed({ removeTapIndicator() }, 5000)
    }

    /**
     * 移除点击指示器
     */
    private fun removeTapIndicator() {
        tapView?.let { windowManager?.removeView(it) }
        tapView = null
    }
}
