package io.sixminutes.ridicule.monitor

import android.Manifest
import android.content.pm.PackageManager
import de.robv.android.xposed.IXposedHookLoadPackage
import de.robv.android.xposed.XC_MethodHook
import de.robv.android.xposed.XposedBridge
import de.robv.android.xposed.XposedHelpers
import de.robv.android.xposed.callbacks.XC_LoadPackage

//  结果输出
//  I  [CarrierMonitor] Called: TelephonyManager.getNetworkOperator → Result: 310260
//     StackTrace:
//       [0] io.sixminutes.ridicule.monitor.CarrierMonitor.access$logCall (CarrierMonitor.kt:15)
//       [1] io.sixminutes.ridicule.monitor.CarrierMonitor$hookTelephonyMethods$1.afterHookedMethod (CarrierMonitor.kt:48)
//       [2] vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//       [3] J.callback (null:292)
//       [4] LSPHooker_.getNetworkOperator (null:8)
//       [5] com.applovin.impl.sdk.k.u (SourceFile:6)
//       [6] com.applovin.impl.sdk.k.z (SourceFile:16)
//       [7] com.applovin.impl.sdk.k.<init> (SourceFile:33)
//       [8] com.applovin.impl.sdk.j.x (SourceFile:1)
//       [9] com.applovin.impl.k5.e (SourceFile:48)
//  I  [CarrierMonitor] Called: TelephonyManager.getNetworkOperator → Result: 310260
//     StackTrace:
//       [0] io.sixminutes.ridicule.monitor.CarrierMonitor.access$logCall (CarrierMonitor.kt:15)
//       [1] io.sixminutes.ridicule.monitor.CarrierMonitor$hookTelephonyMethods$1.afterHookedMethod (CarrierMonitor.kt:48)
//       [2] vTxl.pKyTPAWbQR.SNM.in.XposedBridge$LegacyApiSupport.handleAfter (null:33)
//       [3] J.callback (null:292)
//       [4] LSPHooker_.getNetworkOperator (null:8)
//       [5] com.applovin.impl.sdk.k.v (SourceFile:6)
//       [6] com.applovin.impl.sdk.k.z (SourceFile:17)
//       [7] com.applovin.impl.sdk.k.<init> (SourceFile:33)
//       [8] com.applovin.impl.sdk.j.x (SourceFile:1)
//       [9] com.applovin.impl.k5.e (SourceFile:48)
//  I  [CarrierMonitor] Called: TelephonyManager.getNetworkOperatorName → Result: T-Mobile

class CarrierMonitor : IXposedHookLoadPackage {

    companion object {
        private const val TARGET_PACKAGE = "io.sixminutes.breakingnews"
        private const val LOG_TAG = "CarrierMonitor"
    }

    override fun handleLoadPackage(lpparam: XC_LoadPackage.LoadPackageParam) {
        if (lpparam.packageName != TARGET_PACKAGE) return

        // Hook TelephonyManager 核心方法
        hookTelephonyMethods(lpparam.classLoader)

        // Hook SubscriptionManager（多卡支持）
        hookSubscriptionMethods(lpparam.classLoader)

        // Hook ServiceState（网络状态）
        hookServiceStateMethods(lpparam.classLoader)

        // 绕过权限检查（Android 12+）
        bypassPermissionChecks(lpparam.classLoader)
    }

    private fun hookTelephonyMethods(classLoader: ClassLoader) {
        val telephonyManagerClass = "android.telephony.TelephonyManager"

        // Hook getNetworkOperator()
        XposedHelpers.findAndHookMethod(
            telephonyManagerClass,
            classLoader,
            "getNetworkOperator",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        method = "TelephonyManager.getNetworkOperator",
                        result = param.result,
                        stackDepth = 10 // 记录调用堆栈深度
                    )
                }
            }
        )

        // Hook getNetworkOperatorName()
        XposedHelpers.findAndHookMethod(
            telephonyManagerClass,
            classLoader,
            "getNetworkOperatorName",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        method = "TelephonyManager.getNetworkOperatorName",
                        result = param.result
                    )
                }
            }
        )
    }

    private fun hookSubscriptionMethods(classLoader: ClassLoader) {
        XposedHelpers.findAndHookMethod(
            "android.telephony.SubscriptionManager",
            classLoader,
            "getActiveSubscriptionInfoList",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    val list = param.result as? List<*>
                    list?.forEach { subInfo ->
                        val mcc = XposedHelpers.callMethod(subInfo, "getMcc")
                        val mnc = XposedHelpers.callMethod(subInfo, "getMnc")
                        logCall(
                            method = "SubscriptionManager.getActiveSubscriptionInfoList",
                            result = "MCC=$mcc, MNC=$mnc"
                        )
                    }
                }
            }
        )
    }

    private fun hookServiceStateMethods(classLoader: ClassLoader) {
        XposedHelpers.findAndHookMethod(
            "android.telephony.ServiceState",
            classLoader,
            "getOperatorNumeric",
            object : XC_MethodHook() {
                override fun afterHookedMethod(param: MethodHookParam) {
                    logCall(
                        method = "ServiceState.getOperatorNumeric",
                        result = param.result
                    )
                }
            }
        )
    }

    private fun bypassPermissionChecks(classLoader: ClassLoader) {
        // 绕过 READ_PHONE_STATE 权限检查
        XposedHelpers.findAndHookMethod(
            "android.app.ContextImpl",
            classLoader,
            "checkPermission",
            String::class.java,
            Int::class.java,
            Int::class.java,
            object : XC_MethodHook() {
                override fun beforeHookedMethod(param: MethodHookParam) {
                    if (param.args[0] == Manifest.permission.READ_PHONE_STATE) {
                        param.result = PackageManager.PERMISSION_GRANTED
                    }
                }
            }
        )
    }

    /**
     * 记录调用日志（带堆栈跟踪）
     */
    private fun logCall(method: String, result: Any?, stackDepth: Int = 0) {
        val logMsg = buildString {
            append("[$LOG_TAG] Called: $method → Result: $result")
            if (stackDepth > 0) {
                append("\nStackTrace:")
                Thread.currentThread().stackTrace
                    .drop(3) // 忽略Xposed内部调用
                    .take(stackDepth)
                    .forEachIndexed { i, element ->
                        append("\n  [$i] ${element.className}.${element.methodName} (${element.fileName}:${element.lineNumber})")
                    }
            }
        }

        // 输出到Xposed日志
        XposedBridge.log(logMsg)
    }

}